import textwrap
from typing import List

try:
    import rich  # noqa
    from rich.columns import Columns  # noqa
    from rich.console import RenderableType  # noqa
    from rich.panel import Panel  # noqa

    has_rich = True
except ImportError:
    has_rich = False

from .service import ServiceState


def debug_runner(runner):
    """
    Dump debug information about service state to standard output.

    Call this method from within nameko backdoor which exposes a ``runner``
    local variable.

    If rich_ is available, the output will be way prettier than you'd expect :)

    .. _rich: https://github.com/willmcgugan/rich
    """
    func = debug_state_rich if has_rich else debug_state_simple
    for container in runner.containers:
        service_state = ServiceState.from_container(container)
        func(service_state)


def debug_state_simple(state: ServiceState) -> None:
    """
    Print service state to stdout with some rudimentary formatting.
    """
    print(
        f"Container for service '{state.service_name}' with {len(state.entrypoints)} entrypoints and {len(state.dependencies)} dependencies"
    )
    print(f"Running {state.running_workers}/{state.max_workers} worker threads:")
    for i, worker_state in enumerate(state.worker_states):
        print(
            textwrap.dedent(
                f"""
            ------------
            Thread #{i}: {worker_state.class_name}.{worker_state.method_name}
            Args: {worker_state.args}
            Kwargs: {worker_state.kwargs}
            Context data: {worker_state.data}
            """
            )
        )
        print("".join(worker_state.stacktrace))


def debug_state_rich(state: ServiceState) -> None:
    """
    Pretty-print service state using rich.
    """
    renderables: List[RenderableType] = [
        f"""{len(state.entrypoints)} entrypoints
{len(state.dependencies)} dependencies
Running {state.running_workers}/{state.max_workers} worker threads"""
    ]
    for i, worker_state in enumerate(state.worker_states):
        formatted_stack = "".join(
            f"[bold]{line}[/bold]"
            if "site-packages" not in line
            else f"[dim]{line}[/dim]"
            for line in worker_state.stacktrace
        )
        renderables.append(
            Panel(
                f"""
Args: {worker_state.args}
Kwargs: {worker_state.kwargs}
Context data: {worker_state.data}

Traceback:
{formatted_stack}""",
                title=f"Thread #{i}: [magenta]{worker_state.class_name}.{worker_state.method_name}[/magenta]",
            )
        )
    rich.print(Panel(Columns(renderables), title=f"[bold]{state.service_name}[/bold]"))



from unittest.mock import create_autospec

from pyrabbit.api import Client

from nameko_chassis.discovery import ServiceDiscovery


def test_list_services():
    pyrabbit_client = create_autospec(Client)
    pyrabbit_client.host = "localhost:15672"
    pyrabbit_client.get_queues.return_value = [
        {"name": "rpc-my_service", "vhost": "/"},
        {"name": "celery", "vhost": "/"},
        {"name": "rpc-another_service", "vhost": "/"},
    ]
    discovery = ServiceDiscovery(pyrabbit_client)
    services = discovery.find_services()
    assert services == ["my_service", "another_service"]



#!/usr/bin/env python

from glob import glob
from os.path import basename, splitext

from setuptools import find_packages, setup

with open("README.rst", "r") as f:
    long_description = f.read()


setup(
    name="nameko-chassis",
    version="2.0.1",
    license="Apache-2.0",
    description="nameko-chassis provides an opinionated base class for building resilient, observable microservices. ",
    long_description=long_description,
    author="Emplocity",
    author_email="zbigniew.siciarz@emplocity.pl",
    url="https://github.com/Emplocity/nameko-chassis",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[splitext(basename(path))[0] for path in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Utilities",
    ],
    project_urls={
        "Documentation": "https://nameko-chassis.readthedocs.io/",
        "Changelog": "https://nameko-chassis.readthedocs.io/en/latest/changelog.html",
        "Issue Tracker": "https://github.com/Emplocity/nameko-chassis/issues",
    },
    python_requires=">=3.8.*",
    install_requires=[
        "nameko>=3.0rc11,<4",
        "nameko-sentry>=1.0,<2",
        "nameko-prometheus>=1.5.0,<2",
        "opentelemetry-api>=1.12,<2",
        "opentelemetry-exporter-otlp-proto-http>=1.12,<2",
        "opentelemetry-instrumentation-nameko>=0.3,<1",
        "opentelemetry-sdk>=1.12,<2",
        "opentelemetry-instrumentation-logging>=0.33b0,<1",
        "pyrabbit>=1.1,<2",
        "werkzeug>=1.0,<3",
    ],
)
